@@ -1,11 +1,11 @@
-# @(#)$Id: MailAlias.pm 271 2011-05-30 01:37:52Z pjf $
+# @(#)$Id: MailAlias.pm 285 2011-07-11 12:40:49Z pjf $
 
 package File::MailAlias;
 
 use strict;
 use warnings;
 use namespace::autoclean;
-use version; our $VERSION = qv( sprintf '0.5.%d', q$Rev: 271 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.6.%d', q$Rev: 285 $ =~ /\d+/gmx );
 
 use File::DataClass::Constants;
 use File::DataClass::IO ();
@@ -39,7 +39,8 @@ has 'root_update_attrs' => is => 'ro', isa => 'ArrayRef',
 has '+result_source_attributes' =>
    default                      => sub { return {
       aliases => { attributes => [ qw(comment created owner recipients) ],
-                   defaults   => { comment => [], recipients => [] } }, } };
+                   defaults   => { comment    => [ '-' ],
+                                   recipients => [] } }, } };
 has '+storage_class' =>
    default           => q(+File::MailAlias::Storage);
 
@@ -53,12 +54,25 @@ around BUILDARGS => sub {
    elsif (ref $car eq ARRAY) { $attrs->{path} = $class->catfile( @{ $car } ) }
    else                      { $attrs->{path} = $car.NUL }
 
-   $cdr[0] and $attrs->{system_aliases} = $cdr[0];
-   $cdr[1] and $attrs->{newaliases    } = [ $cdr[1] ];
+   $cdr[ 0 ] and $attrs->{system_aliases} =   $cdr[ 0 ];
+   $cdr[ 1 ] and $attrs->{newaliases    } = [ $cdr[ 1 ] ];
 
    return $attrs;
 };
 
+{  my $map = {}; my $mtime = 0;
+
+   sub aliases_map {
+      my $self = shift; my (undef, $meta) = $self->cache->get( $self->path );
+
+      ($meta and defined $meta->{mtime}) or $meta = { mtime => 1 };
+      $meta->{mtime} > $mtime and $mtime = $meta->{mtime}
+         and $map = { map { $_ => TRUE } @{ $self->list( 'nobody' )->list } };
+
+      return $map;
+   }
+}
+
 sub create {
    my ($self, $args) = @_;
 
@@ -101,7 +115,7 @@ sub update {
 sub update_as_root {
    my $self = shift; my $cmd = join SPC, @{ $self->newaliases || [] };
 
-   ($self->newaliases and $cmd = can_run( $self->newaliases->[0] ))
+   ($self->newaliases and $cmd = can_run( $self->newaliases->[ 0 ] ))
       or $self->throw( error => 'Path [_1] cannot execute', args => [ $cmd ] );
 
    copy( NUL.$self->path, $self->catfile( $self->system_aliases ) )
@@ -161,7 +175,7 @@ File::MailAlias - Domain model for the system mail aliases file
 
 =head1 Version
 
-0.5.$Revision: 271 $
+0.6.$Revision: 285 $
 
 =head1 Synopsis
 
@@ -213,6 +227,13 @@ access to the real mail alias file
 
 =head2 BUILD
 
+=head2 aliases_map
+
+   $alias_obj->aliases_map;
+
+Returns a hash ref of aliases. Caches the result and updates automatically
+by reading the cache mod time
+
 =head2 create
 
    $alias_obj->create( { name => $name, fields => $fields } );
